# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import ad_type_infos
from google.ads.googleads.v20.common.types import criteria
from google.ads.googleads.v20.resources.types import (
    keyword_theme_constant as gagr_keyword_theme_constant,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "SuggestSmartCampaignBudgetOptionsRequest",
        "SmartCampaignSuggestionInfo",
        "SuggestSmartCampaignBudgetOptionsResponse",
        "SuggestSmartCampaignAdRequest",
        "SuggestSmartCampaignAdResponse",
        "SuggestKeywordThemesRequest",
        "SuggestKeywordThemesResponse",
    },
)


class SuggestSmartCampaignBudgetOptionsRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSuggestService.SuggestSmartCampaignBudgetOptions][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestSmartCampaignBudgetOptions].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose budget
            options are to be suggested.
        campaign (str):
            Required. The resource name of the campaign
            to get suggestion for.

            This field is a member of `oneof`_ ``suggestion_data``.
        suggestion_info (google.ads.googleads.v20.services.types.SmartCampaignSuggestionInfo):
            Required. Information needed to get budget
            options

            This field is a member of `oneof`_ ``suggestion_data``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="suggestion_data",
    )
    suggestion_info: "SmartCampaignSuggestionInfo" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="suggestion_data",
        message="SmartCampaignSuggestionInfo",
    )


class SmartCampaignSuggestionInfo(proto.Message):
    r"""Information needed to get suggestion for Smart Campaign. More
    information provided will help the system to derive better
    suggestions.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        final_url (str):
            Optional. Landing page URL of the campaign.
        language_code (str):
            Optional. The two letter advertising language
            for the Smart campaign to be constructed,
            default to 'en' if not set.
        ad_schedules (MutableSequence[google.ads.googleads.v20.common.types.AdScheduleInfo]):
            Optional. The business ad schedule.
        keyword_themes (MutableSequence[google.ads.googleads.v20.common.types.KeywordThemeInfo]):
            Optional. Smart campaign keyword themes. This
            field may greatly improve suggestion accuracy
            and we recommend always setting it if possible.
        business_context (google.ads.googleads.v20.services.types.SmartCampaignSuggestionInfo.BusinessContext):
            Optional. Context describing the business to
            advertise.

            This field is a member of `oneof`_ ``business_setting``.
        business_profile_location (str):
            Optional. The resource name of a Business Profile location.
            Business Profile location resource names can be fetched
            through the Business Profile API and adhere to the following
            format: ``locations/{locationId}``.

            See the [Business Profile API]
            (https://developers.google.com/my-business/reference/businessinformation/rest/v1/accounts.locations)
            for additional details.

            This field is a member of `oneof`_ ``business_setting``.
        location_list (google.ads.googleads.v20.services.types.SmartCampaignSuggestionInfo.LocationList):
            Optional. The targeting geo location by
            locations.

            This field is a member of `oneof`_ ``geo_target``.
        proximity (google.ads.googleads.v20.common.types.ProximityInfo):
            Optional. The targeting geo location by
            proximity.

            This field is a member of `oneof`_ ``geo_target``.
    """

    class LocationList(proto.Message):
        r"""A list of locations.

        Attributes:
            locations (MutableSequence[google.ads.googleads.v20.common.types.LocationInfo]):
                Required. Locations.
        """

        locations: MutableSequence[criteria.LocationInfo] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.LocationInfo,
        )

    class BusinessContext(proto.Message):
        r"""A context that describes a business.

        Attributes:
            business_name (str):
                Optional. The name of the business.
        """

        business_name: str = proto.Field(
            proto.STRING,
            number=1,
        )

    final_url: str = proto.Field(
        proto.STRING,
        number=1,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=3,
    )
    ad_schedules: MutableSequence[criteria.AdScheduleInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=criteria.AdScheduleInfo,
        )
    )
    keyword_themes: MutableSequence[criteria.KeywordThemeInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=7,
            message=criteria.KeywordThemeInfo,
        )
    )
    business_context: BusinessContext = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="business_setting",
        message=BusinessContext,
    )
    business_profile_location: str = proto.Field(
        proto.STRING,
        number=9,
        oneof="business_setting",
    )
    location_list: LocationList = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="geo_target",
        message=LocationList,
    )
    proximity: criteria.ProximityInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="geo_target",
        message=criteria.ProximityInfo,
    )


class SuggestSmartCampaignBudgetOptionsResponse(proto.Message):
    r"""Response message for
    [SmartCampaignSuggestService.SuggestSmartCampaignBudgetOptions][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestSmartCampaignBudgetOptions].
    Depending on whether the system could suggest the options, either
    all of the options or none of them might be returned.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        low (google.ads.googleads.v20.services.types.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption):
            Optional. The lowest budget option.

            This field is a member of `oneof`_ ``_low``.
        recommended (google.ads.googleads.v20.services.types.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption):
            Optional. The recommended budget option.

            This field is a member of `oneof`_ ``_recommended``.
        high (google.ads.googleads.v20.services.types.SuggestSmartCampaignBudgetOptionsResponse.BudgetOption):
            Optional. The highest budget option.

            This field is a member of `oneof`_ ``_high``.
    """

    class Metrics(proto.Message):
        r"""Performance metrics for a given budget option.

        Attributes:
            min_daily_clicks (int):
                The estimated min daily clicks.
            max_daily_clicks (int):
                The estimated max daily clicks.
        """

        min_daily_clicks: int = proto.Field(
            proto.INT64,
            number=1,
        )
        max_daily_clicks: int = proto.Field(
            proto.INT64,
            number=2,
        )

    class BudgetOption(proto.Message):
        r"""Smart Campaign budget option.

        Attributes:
            daily_amount_micros (int):
                The amount of the budget, in the local
                currency for the account. Amount is specified in
                micros, where one million is equivalent to one
                currency unit.
            metrics (google.ads.googleads.v20.services.types.SuggestSmartCampaignBudgetOptionsResponse.Metrics):
                Metrics pertaining to the suggested budget,
                could be empty if there is not enough
                information to derive the estimates.
        """

        daily_amount_micros: int = proto.Field(
            proto.INT64,
            number=1,
        )
        metrics: "SuggestSmartCampaignBudgetOptionsResponse.Metrics" = (
            proto.Field(
                proto.MESSAGE,
                number=2,
                message="SuggestSmartCampaignBudgetOptionsResponse.Metrics",
            )
        )

    low: BudgetOption = proto.Field(
        proto.MESSAGE,
        number=1,
        optional=True,
        message=BudgetOption,
    )
    recommended: BudgetOption = proto.Field(
        proto.MESSAGE,
        number=2,
        optional=True,
        message=BudgetOption,
    )
    high: BudgetOption = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message=BudgetOption,
    )


class SuggestSmartCampaignAdRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSuggestService.SuggestSmartCampaignAd][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestSmartCampaignAd].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        suggestion_info (google.ads.googleads.v20.services.types.SmartCampaignSuggestionInfo):
            Required. Inputs used to suggest a Smart campaign ad.
            Required fields: final_url, language_code, keyword_themes.
            Optional but recommended fields to improve the quality of
            the suggestion: business_setting and geo_target.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    suggestion_info: "SmartCampaignSuggestionInfo" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="SmartCampaignSuggestionInfo",
    )


class SuggestSmartCampaignAdResponse(proto.Message):
    r"""Response message for
    [SmartCampaignSuggestService.SuggestSmartCampaignAd][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestSmartCampaignAd].

    Attributes:
        ad_info (google.ads.googleads.v20.common.types.SmartCampaignAdInfo):
            Optional. Ad info includes 3 creative
            headlines and 2 creative descriptions.
    """

    ad_info: ad_type_infos.SmartCampaignAdInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=ad_type_infos.SmartCampaignAdInfo,
    )


class SuggestKeywordThemesRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSuggestService.SuggestKeywordThemes][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestKeywordThemes].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        suggestion_info (google.ads.googleads.v20.services.types.SmartCampaignSuggestionInfo):
            Required. Information to get keyword theme suggestions.
            Required fields:

            -  suggestion_info.final_url
            -  suggestion_info.language_code
            -  suggestion_info.geo_target

            Recommended fields:

            -  suggestion_info.business_setting
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    suggestion_info: "SmartCampaignSuggestionInfo" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="SmartCampaignSuggestionInfo",
    )


class SuggestKeywordThemesResponse(proto.Message):
    r"""Response message for
    [SmartCampaignSuggestService.SuggestKeywordThemes][google.ads.googleads.v20.services.SmartCampaignSuggestService.SuggestKeywordThemes].

    Attributes:
        keyword_themes (MutableSequence[google.ads.googleads.v20.services.types.SuggestKeywordThemesResponse.KeywordTheme]):
            Smart campaign keyword theme suggestions.
    """

    class KeywordTheme(proto.Message):
        r"""A Smart campaign keyword theme suggestion.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            keyword_theme_constant (google.ads.googleads.v20.resources.types.KeywordThemeConstant):
                A Smart campaign keyword theme constant.

                This field is a member of `oneof`_ ``keyword_theme``.
            free_form_keyword_theme (str):
                A free-form text keyword theme.

                This field is a member of `oneof`_ ``keyword_theme``.
        """

        keyword_theme_constant: (
            gagr_keyword_theme_constant.KeywordThemeConstant
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="keyword_theme",
            message=gagr_keyword_theme_constant.KeywordThemeConstant,
        )
        free_form_keyword_theme: str = proto.Field(
            proto.STRING,
            number=2,
            oneof="keyword_theme",
        )

    keyword_themes: MutableSequence[KeywordTheme] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=KeywordTheme,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    placement_type as gage_placement_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "PerformanceMaxPlacementView",
    },
)


class PerformanceMaxPlacementView(proto.Message):
    r"""A view with impression metrics for Performance Max campaign
    placements.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the Performance Max
            placement view. Performance Max placement view resource
            names have the form:

            ``customers/{customer_id}/performanceMaxPlacementViews/{base_64_placement}``
        placement (str):
            Output only. The default placement string,
            such as the website URL, mobile application ID,
            or a YouTube video ID.

            This field is a member of `oneof`_ ``_placement``.
        display_name (str):
            Output only. The name displayed to represent
            the placement, such as the URL name for
            websites, YouTube video name for YouTube videos,
            and translated mobile app name for mobile apps.

            This field is a member of `oneof`_ ``_display_name``.
        target_url (str):
            Output only. URL of the placement, for
            example, website, link to the mobile application
            in app store, or a YouTube video URL.

            This field is a member of `oneof`_ ``_target_url``.
        placement_type (google.ads.googleads.v20.enums.types.PlacementTypeEnum.PlacementType):
            Output only. Type of the placement. Possible values for
            Performance Max placements are WEBSITE, MOBILE_APPLICATION,
            or YOUTUBE_VIDEO.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    placement: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    target_url: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    placement_type: gage_placement_type.PlacementTypeEnum.PlacementType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=gage_placement_type.PlacementTypeEnum.PlacementType,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CampaignCriterionErrorEnum",
    },
)


class CampaignCriterionErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign criterion
    errors.

    """

    class CampaignCriterionError(proto.Enum):
        r"""Enum describing possible campaign criterion errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CONCRETE_TYPE_REQUIRED (2):
                Concrete type of criterion (keyword v.s.
                placement) is required for CREATE and UPDATE
                operations.
            INVALID_PLACEMENT_URL (3):
                Invalid placement URL.
            CANNOT_EXCLUDE_CRITERIA_TYPE (4):
                Criteria type can not be excluded for the
                campaign by the customer. like AOL account type
                cannot target site type criteria
            CANNOT_SET_STATUS_FOR_CRITERIA_TYPE (5):
                Cannot set the campaign criterion status for
                this criteria type.
            CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA (6):
                Cannot set the campaign criterion status for
                an excluded criteria.
            CANNOT_TARGET_AND_EXCLUDE (7):
                Cannot target and exclude the same criterion.
            TOO_MANY_OPERATIONS (8):
                The mutate contained too many operations.
            OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE (9):
                This operator cannot be applied to a
                criterion of this type.
            SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL (10):
                The Shopping campaign sales country is not
                supported for ProductSalesChannel targeting.
            CANNOT_ADD_EXISTING_FIELD (11):
                The existing field can't be updated with
                CREATE operation. It can be updated with UPDATE
                operation only.
            CANNOT_UPDATE_NEGATIVE_CRITERION (12):
                Negative criteria are immutable, so updates
                are not allowed.
            CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION (13):
                Only free form names are allowed for negative
                Smart campaign keyword theme.
            INVALID_KEYWORD_THEME_CONSTANT (14):
                Invalid Smart campaign keyword theme constant
                criterion.
            MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME (15):
                A Smart campaign keyword theme constant or
                free-form Smart campaign keyword theme is
                required.
            CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN (16):
                A Smart campaign may not target proximity and
                location criteria simultaneously.
            CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN (17):
                A Smart campaign may not target multiple
                proximity criteria.
            LOCATION_NOT_LAUNCHED_FOR_LOCAL_SERVICES_CAMPAIGN (18):
                Location is not launched for Local Services
                Campaigns.
            LOCATION_INVALID_FOR_LOCAL_SERVICES_CAMPAIGN (19):
                A Local Services campaign may not target
                certain criteria types.
            CANNOT_TARGET_COUNTRY_FOR_LOCAL_SERVICES_CAMPAIGN (20):
                Country locations are not supported for Local
                Services campaign.
            LOCATION_NOT_IN_HOME_COUNTRY_FOR_LOCAL_SERVICES_CAMPAIGN (21):
                Location is not within the home country of
                Local Services campaign.
            CANNOT_ADD_OR_REMOVE_LOCATION_FOR_LOCAL_SERVICES_CAMPAIGN (22):
                Local Services profile does not exist for a
                particular Local Services campaign.
            AT_LEAST_ONE_POSITIVE_LOCATION_REQUIRED_FOR_LOCAL_SERVICES_CAMPAIGN (23):
                Local Services campaign must have at least
                one target location.
            AT_LEAST_ONE_LOCAL_SERVICE_ID_CRITERION_REQUIRED_FOR_LOCAL_SERVICES_CAMPAIGN (24):
                At least one positive local service ID
                criterion is required for a Local Services
                campaign.
            LOCAL_SERVICE_ID_NOT_FOUND_FOR_CATEGORY (25):
                Local service ID is not found under selected
                categories in local services campaign setting.
            CANNOT_ATTACH_BRAND_LIST_TO_NON_QUALIFIED_SEARCH_CAMPAIGN (26):
                For search advertising channel, brand lists
                can only be applied to exclusive targeting,
                broad match campaigns for inclusive targeting or
                PMax generated campaigns.
            CANNOT_REMOVE_ALL_LOCATIONS_DUE_TO_TOO_MANY_COUNTRY_EXCLUSIONS (27):
                Campaigns that target all countries and
                territories are limited to a certain number of
                top-level location exclusions. If removing a
                criterion causes the campaign to target all
                countries and territories and the campaign has
                more top-level location exclusions than the
                limit allows, then this error is returned.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONCRETE_TYPE_REQUIRED = 2
        INVALID_PLACEMENT_URL = 3
        CANNOT_EXCLUDE_CRITERIA_TYPE = 4
        CANNOT_SET_STATUS_FOR_CRITERIA_TYPE = 5
        CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA = 6
        CANNOT_TARGET_AND_EXCLUDE = 7
        TOO_MANY_OPERATIONS = 8
        OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE = 9
        SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL = 10
        CANNOT_ADD_EXISTING_FIELD = 11
        CANNOT_UPDATE_NEGATIVE_CRITERION = 12
        CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION = 13
        INVALID_KEYWORD_THEME_CONSTANT = 14
        MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME = 15
        CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN = (
            16
        )
        CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN = 17
        LOCATION_NOT_LAUNCHED_FOR_LOCAL_SERVICES_CAMPAIGN = 18
        LOCATION_INVALID_FOR_LOCAL_SERVICES_CAMPAIGN = 19
        CANNOT_TARGET_COUNTRY_FOR_LOCAL_SERVICES_CAMPAIGN = 20
        LOCATION_NOT_IN_HOME_COUNTRY_FOR_LOCAL_SERVICES_CAMPAIGN = 21
        CANNOT_ADD_OR_REMOVE_LOCATION_FOR_LOCAL_SERVICES_CAMPAIGN = 22
        AT_LEAST_ONE_POSITIVE_LOCATION_REQUIRED_FOR_LOCAL_SERVICES_CAMPAIGN = 23
        AT_LEAST_ONE_LOCAL_SERVICE_ID_CRITERION_REQUIRED_FOR_LOCAL_SERVICES_CAMPAIGN = (
            24
        )
        LOCAL_SERVICE_ID_NOT_FOUND_FOR_CATEGORY = 25
        CANNOT_ATTACH_BRAND_LIST_TO_NON_QUALIFIED_SEARCH_CAMPAIGN = 26
        CANNOT_REMOVE_ALL_LOCATIONS_DUE_TO_TOO_MANY_COUNTRY_EXCLUSIONS = 27


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ConversionOriginEnum",
    },
)


class ConversionOriginEnum(proto.Message):
    r"""Container for enum describing possible conversion origins."""

    class ConversionOrigin(proto.Enum):
        r"""The possible places where a conversion can occur.

        Values:
            UNSPECIFIED (0):
                The conversion origin has not been specified.
            UNKNOWN (1):
                The conversion origin is not known in this
                version.
            WEBSITE (2):
                Conversion that occurs when a user visits a
                website or takes an action there after viewing
                an ad.
            GOOGLE_HOSTED (3):
                Conversions reported by an offline pipeline
                which collects local actions from Google-hosted
                pages (for example, Google Maps, Google Place
                Page, etc) and attributes them to relevant ad
                events.
            APP (4):
                Conversion that occurs when a user performs
                an action through any app platforms.
            CALL_FROM_ADS (5):
                Conversion that occurs when a user makes a
                call from ads.
            STORE (6):
                Conversion that occurs when a user visits or
                makes a purchase at a physical store.
            YOUTUBE_HOSTED (7):
                Conversion that occurs on YouTube.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        WEBSITE = 2
        GOOGLE_HOSTED = 3
        APP = 4
        CALL_FROM_ADS = 5
        STORE = 6
        YOUTUBE_HOSTED = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import dates
from google.ads.googleads.v22.enums.types import keyword_plan_forecast_interval


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "KeywordPlan",
        "KeywordPlanForecastPeriod",
    },
)


class KeywordPlan(proto.Message):
    r"""A Keyword Planner plan.
    Max number of saved keyword plans: 10000.
    It's possible to remove plans if limit is reached.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Keyword Planner plan.
            KeywordPlan resource names have the form:

            ``customers/{customer_id}/keywordPlans/{kp_plan_id}``
        id (int):
            Output only. The ID of the keyword plan.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the keyword plan.

            This field is required and should not be empty
            when creating new keyword plans.

            This field is a member of `oneof`_ ``_name``.
        forecast_period (google.ads.googleads.v22.resources.types.KeywordPlanForecastPeriod):
            The date period used for forecasting the
            plan.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    forecast_period: "KeywordPlanForecastPeriod" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="KeywordPlanForecastPeriod",
    )


class KeywordPlanForecastPeriod(proto.Message):
    r"""The forecasting period associated with the keyword plan.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        date_interval (google.ads.googleads.v22.enums.types.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval):
            A future date range relative to the current
            date used for forecasting.

            This field is a member of `oneof`_ ``interval``.
        date_range (google.ads.googleads.v22.common.types.DateRange):
            The custom date range used for forecasting.
            It cannot be greater than a year.
            The start and end dates must be in the future.
            Otherwise, an error will be returned when the
            forecasting action is performed. The start and
            end dates are inclusive.

            This field is a member of `oneof`_ ``interval``.
    """

    date_interval: (
        keyword_plan_forecast_interval.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval
    ) = proto.Field(
        proto.ENUM,
        number=1,
        oneof="interval",
        enum=keyword_plan_forecast_interval.KeywordPlanForecastIntervalEnum.KeywordPlanForecastInterval,
    )
    date_range: dates.DateRange = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="interval",
        message=dates.DateRange,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "CallToActionTypeEnum",
    },
)


class CallToActionTypeEnum(proto.Message):
    r"""Container for enum describing the call to action types."""

    class CallToActionType(proto.Enum):
        r"""Enum describing possible types of call to action.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            LEARN_MORE (2):
                The call to action type is learn more.
            GET_QUOTE (3):
                The call to action type is get quote.
            APPLY_NOW (4):
                The call to action type is apply now.
            SIGN_UP (5):
                The call to action type is sign up.
            CONTACT_US (6):
                The call to action type is contact us.
            SUBSCRIBE (7):
                The call to action type is subscribe.
            DOWNLOAD (8):
                The call to action type is download.
            BOOK_NOW (9):
                The call to action type is book now.
            SHOP_NOW (10):
                The call to action type is shop now.
            BUY_NOW (11):
                The call to action type is buy now.
            DONATE_NOW (12):
                The call to action type is donate now.
            ORDER_NOW (13):
                The call to action type is order now.
            PLAY_NOW (14):
                The call to action type is play now.
            SEE_MORE (15):
                The call to action type is see more.
            START_NOW (16):
                The call to action type is start now.
            VISIT_SITE (17):
                The call to action type is visit site.
            WATCH_NOW (18):
                The call to action type is watch now.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LEARN_MORE = 2
        GET_QUOTE = 3
        APPLY_NOW = 4
        SIGN_UP = 5
        CONTACT_US = 6
        SUBSCRIBE = 7
        DOWNLOAD = 8
        BOOK_NOW = 9
        SHOP_NOW = 10
        BUY_NOW = 11
        DONATE_NOW = 12
        ORDER_NOW = 13
        PLAY_NOW = 14
        SEE_MORE = 15
        START_NOW = 16
        VISIT_SITE = 17
        WATCH_NOW = 18


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import custom_audience_member_type
from google.ads.googleads.v21.enums.types import custom_audience_status
from google.ads.googleads.v21.enums.types import custom_audience_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomAudience",
        "CustomAudienceMember",
    },
)


class CustomAudience(proto.Message):
    r"""A custom audience. This is a list of users by interest. The unique
    key of a custom audience consists of the following fields: name.
    Violating the unique key constraint produces error:
    CustomAudienceError.NAME_ALREADY_USED

    Attributes:
        resource_name (str):
            Immutable. The resource name of the custom audience. Custom
            audience resource names have the form:

            ``customers/{customer_id}/customAudiences/{custom_audience_id}``
        id (int):
            Output only. ID of the custom audience.
        status (google.ads.googleads.v21.enums.types.CustomAudienceStatusEnum.CustomAudienceStatus):
            Output only. Status of this custom audience.
            Indicates whether the custom audience is enabled
            or removed.
        name (str):
            Name of the custom audience. It should be
            unique for all custom audiences created by a
            customer. This field is required for creating
            operations.
        type_ (google.ads.googleads.v21.enums.types.CustomAudienceTypeEnum.CustomAudienceType):
            Type of the custom audience. ("INTEREST" OR
            "PURCHASE_INTENT" is not allowed for newly created custom
            audience but kept for existing audiences)
        description (str):
            Description of this custom audience.
        members (MutableSequence[google.ads.googleads.v21.resources.types.CustomAudienceMember]):
            List of custom audience members that this
            custom audience is composed of. Members can be
            added during CustomAudience creation. If members
            are presented in UPDATE operation, existing
            members will be overridden.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    status: (
        custom_audience_status.CustomAudienceStatusEnum.CustomAudienceStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=custom_audience_status.CustomAudienceStatusEnum.CustomAudienceStatus,
    )
    name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    type_: custom_audience_type.CustomAudienceTypeEnum.CustomAudienceType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=custom_audience_type.CustomAudienceTypeEnum.CustomAudienceType,
        )
    )
    description: str = proto.Field(
        proto.STRING,
        number=6,
    )
    members: MutableSequence["CustomAudienceMember"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="CustomAudienceMember",
    )


class CustomAudienceMember(proto.Message):
    r"""A member of custom audience. A member can be a KEYWORD, URL,
    PLACE_CATEGORY or APP. It can only be created or removed but not
    changed.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        member_type (google.ads.googleads.v21.enums.types.CustomAudienceMemberTypeEnum.CustomAudienceMemberType):
            The type of custom audience member, KEYWORD, URL,
            PLACE_CATEGORY or APP.
        keyword (str):
            A keyword or keyword phrase — at most 10
            words and 80 characters. Languages with
            double-width characters such as Chinese,
            Japanese, or Korean, are allowed 40 characters,
            which describes the user's interests or actions.

            This field is a member of `oneof`_ ``value``.
        url (str):
            An HTTP URL, protocol-included — at most 2048
            characters, which includes contents users have
            interests in.

            This field is a member of `oneof`_ ``value``.
        place_category (int):
            A place type described by a place category
            users visit.

            This field is a member of `oneof`_ ``value``.
        app (str):
            A package name of Android apps which users
            installed such as com.google.example.

            This field is a member of `oneof`_ ``value``.
    """

    member_type: (
        custom_audience_member_type.CustomAudienceMemberTypeEnum.CustomAudienceMemberType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=custom_audience_member_type.CustomAudienceMemberTypeEnum.CustomAudienceMemberType,
    )
    keyword: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="value",
    )
    url: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="value",
    )
    place_category: int = proto.Field(
        proto.INT64,
        number=4,
        oneof="value",
    )
    app: str = proto.Field(
        proto.STRING,
        number=5,
        oneof="value",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.resources.types import billing_setup


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "MutateBillingSetupRequest",
        "BillingSetupOperation",
        "MutateBillingSetupResponse",
        "MutateBillingSetupResult",
    },
)


class MutateBillingSetupRequest(proto.Message):
    r"""Request message for billing setup mutate operations.

    Attributes:
        customer_id (str):
            Required. Id of the customer to apply the
            billing setup mutate operation to.
        operation (google.ads.googleads.v21.services.types.BillingSetupOperation):
            Required. The operation to perform.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "BillingSetupOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="BillingSetupOperation",
    )


class BillingSetupOperation(proto.Message):
    r"""A single operation on a billing setup, which describes the
    cancellation of an existing billing setup.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v21.resources.types.BillingSetup):
            Creates a billing setup. No resource name is
            expected for the new billing setup.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Resource name of the billing setup to remove. A setup cannot
            be removed unless it is in a pending state or its scheduled
            start time is in the future. The resource name looks like
            ``customers/{customer_id}/billingSetups/{billing_id}``.

            This field is a member of `oneof`_ ``operation``.
    """

    create: billing_setup.BillingSetup = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=billing_setup.BillingSetup,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="operation",
    )


class MutateBillingSetupResponse(proto.Message):
    r"""Response message for a billing setup operation.

    Attributes:
        result (google.ads.googleads.v21.services.types.MutateBillingSetupResult):
            A result that identifies the resource
            affected by the mutate request.
    """

    result: "MutateBillingSetupResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="MutateBillingSetupResult",
    )


class MutateBillingSetupResult(proto.Message):
    r"""Result for a single billing setup mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    advertising_channel_type as gage_advertising_channel_type,
)
from google.ads.googleads.v22.enums.types import (
    asset_field_type as gage_asset_field_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "GenerateTextRequest",
        "GenerateTextResponse",
        "GeneratedText",
        "GenerateImagesRequest",
        "FinalUrlImageGenerationInput",
        "FreeformImageGenerationInput",
        "ProductRecontextGenerationImageInput",
        "GenerateImagesResponse",
        "SourceImage",
        "GeneratedImage",
        "AssetGenerationExistingContext",
    },
)


class GenerateTextRequest(proto.Message):
    r"""Request message for
    [AssetGenerationService.GenerateText][google.ads.googleads.v22.services.AssetGenerationService.GenerateText]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer to generate
            assets for. Required.
        asset_field_types (MutableSequence[google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType]):
            Required. Which field types text is being generated for.
            Supported values are: HEADLINE, LONG_HEADLINE, DESCRIPTION.
            Required.
        final_url (str):
            Optional. Final url to use as a source for generating
            assets. Required if existing_generation_context is not
            provided or does not have a final url associated with it.
        freeform_prompt (str):
            Optional. A freeform description of what
            assets should be generated. The string length
            must be between 1 and 1500, inclusive.
        keywords (MutableSequence[str]):
            Optional. A freeform list of keywords that
            are relevant, used to inform asset generation.
        existing_generation_context (google.ads.googleads.v22.services.types.AssetGenerationExistingContext):
            Optional. The setting for which assets are
            being generated, such as an existing AssetGroup
            or AdGroupAd.

            This field is a member of `oneof`_ ``context``.
        advertising_channel_type (google.ads.googleads.v22.enums.types.AdvertisingChannelTypeEnum.AdvertisingChannelType):
            Optional. The advertising channel for which to generate
            assets. Required if existing_context is not provided.

            Supported channel types: SEARCH, PERFORMANCE_MAX, DISPLAY,
            and DEMAND_GEN

            This field is a member of `oneof`_ ``context``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_types: MutableSequence[
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    final_url: str = proto.Field(
        proto.STRING,
        number=5,
    )
    freeform_prompt: str = proto.Field(
        proto.STRING,
        number=6,
    )
    keywords: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    existing_generation_context: "AssetGenerationExistingContext" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="context",
        message="AssetGenerationExistingContext",
    )
    advertising_channel_type: (
        gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        oneof="context",
        enum=gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType,
    )


class GenerateTextResponse(proto.Message):
    r"""Response message for
    [AssetGenerationService.GenerateText][google.ads.googleads.v22.services.AssetGenerationService.GenerateText]

    Attributes:
        generated_text (MutableSequence[google.ads.googleads.v22.services.types.GeneratedText]):
            List of text that was generated and the field
            type to use it as.
    """

    generated_text: MutableSequence["GeneratedText"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="GeneratedText",
    )


class GeneratedText(proto.Message):
    r"""Data and metadata about a piece of generated text.

    Attributes:
        text (str):
            A string of text that was generated.
        asset_field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The type of asset this text is intended to be
            used as.
    """

    text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


class GenerateImagesRequest(proto.Message):
    r"""Request message for
    [AssetGenerationService.GenerateImages][google.ads.googleads.v22.services.AssetGenerationService.GenerateImages]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer for whom the
            images are being generated. Required.
        asset_field_types (MutableSequence[google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType]):
            Optional. Output field types for generated images. Supported
            values are MARKETING_IMAGE, SQUARE_MARKETING_IMAGE,
            PORTRAIT_MARKETING_IMAGE, and TALL_PORTRAIT_MARKETING_IMAGE.
            All specified field types must be compatible with the
            ``advertising_channel_type`` or
            ``existing_generation_context`` (whichever is set). If no
            field types are provided, images will be generated for all
            compatible field types.
        advertising_channel_type (google.ads.googleads.v22.enums.types.AdvertisingChannelTypeEnum.AdvertisingChannelType):
            Optional. The advertising channel type for which the images
            are being generated. This field is required if
            ``existing_generation_context`` is not provided. Supported
            channel types include SEARCH, PERFORMANCE_MAX, DISPLAY, and
            DEMAND_GEN.

            This field is a member of `oneof`_ ``context``.
        final_url_generation (google.ads.googleads.v22.services.types.FinalUrlImageGenerationInput):
            Optional. Generate images from a final url.

            This field is a member of `oneof`_ ``generation_type``.
        freeform_generation (google.ads.googleads.v22.services.types.FreeformImageGenerationInput):
            Optional. Generate images from a freeform
            prompt.

            This field is a member of `oneof`_ ``generation_type``.
        product_recontext_generation (google.ads.googleads.v22.services.types.ProductRecontextGenerationImageInput):
            Optional. Generate new images by
            recontextualizing existing product images.

            This field is a member of `oneof`_ ``generation_type``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_types: MutableSequence[
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    advertising_channel_type: (
        gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        oneof="context",
        enum=gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType,
    )
    final_url_generation: "FinalUrlImageGenerationInput" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="generation_type",
        message="FinalUrlImageGenerationInput",
    )
    freeform_generation: "FreeformImageGenerationInput" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="generation_type",
        message="FreeformImageGenerationInput",
    )
    product_recontext_generation: "ProductRecontextGenerationImageInput" = (
        proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="generation_type",
            message="ProductRecontextGenerationImageInput",
        )
    )


class FinalUrlImageGenerationInput(proto.Message):
    r"""Input for guiding image asset generation with a final url.

    Attributes:
        final_url (str):
            Required. A final url to guide the image
            generation process. Required.
    """

    final_url: str = proto.Field(
        proto.STRING,
        number=1,
    )


class FreeformImageGenerationInput(proto.Message):
    r"""Input for guiding image asset generation with a freeform
    prompt.

    Attributes:
        freeform_prompt (str):
            Required. A freeform text description to
            guide the image generation process. The maximum
            length is 1500 characters. Required.
    """

    freeform_prompt: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ProductRecontextGenerationImageInput(proto.Message):
    r"""Input for generating new images by recontextualizing existing
    product images.

    Attributes:
        prompt (str):
            Optional. A freeform description of the
            assets to be generated. Maximum character limit
            is 1500.
        source_images (MutableSequence[google.ads.googleads.v22.services.types.SourceImage]):
            Required. Product images to use for
            generating new images. 1-3 images must be
            provided.
    """

    prompt: str = proto.Field(
        proto.STRING,
        number=1,
    )
    source_images: MutableSequence["SourceImage"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="SourceImage",
    )


class GenerateImagesResponse(proto.Message):
    r"""Response message for
    [AssetGenerationService.GenerateImages][google.ads.googleads.v22.services.AssetGenerationService.GenerateImages]

    Attributes:
        generated_images (MutableSequence[google.ads.googleads.v22.services.types.GeneratedImage]):
            Successfully generated images.
    """

    generated_images: MutableSequence["GeneratedImage"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="GeneratedImage",
    )


class SourceImage(proto.Message):
    r"""A source image to be used in the generation process.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        image_data (bytes):
            Optional. Raw bytes of the image to use.

            This field is a member of `oneof`_ ``image``.
    """

    image_data: bytes = proto.Field(
        proto.BYTES,
        number=1,
        oneof="image",
    )


class GeneratedImage(proto.Message):
    r"""Generated image data and metadata.

    Attributes:
        image_temporary_url (str):
            A temporary URL for the generated image.
        asset_field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The intended field type for this generated
            image.
    """

    image_temporary_url: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


class AssetGenerationExistingContext(proto.Message):
    r"""The context for which assets are being generated, such as an
    existing AssetGroup or AdGroupAd.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        existing_asset_group (str):
            Resource name of an existing AssetGroup for
            which these assets are being generated.

            This field is a member of `oneof`_ ``existing_context``.
        existing_ad_group_ad (str):
            Resource name of an existing AdGroupAd for
            which these assets are being generated.

            This field is a member of `oneof`_ ``existing_context``.
    """

    existing_asset_group: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="existing_context",
    )
    existing_ad_group_ad: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="existing_context",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    conversion_custom_variable_status,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionCustomVariable",
    },
)


class ConversionCustomVariable(proto.Message):
    r"""A conversion custom variable
    See "About custom variables for conversions" at
    https://support.google.com/google-ads/answer/9964350

    Attributes:
        resource_name (str):
            Immutable. The resource name of the conversion custom
            variable. Conversion custom variable resource names have the
            form:

            ``customers/{customer_id}/conversionCustomVariables/{conversion_custom_variable_id}``
        id (int):
            Output only. The ID of the conversion custom
            variable.
        name (str):
            Required. The name of the conversion custom
            variable. Name should be unique. The maximum
            length of name is 100 characters. There should
            not be any extra spaces before and after.
        tag (str):
            Required. Immutable. The tag of the
            conversion custom variable. It is used in the
            event snippet and sent to Google Ads along with
            conversion pings. For conversion uploads in
            Google Ads API, the resource name of the
            conversion custom variable is used. Tag should
            be unique. The maximum size of tag is 100 bytes.
            There should not be any extra spaces before and
            after. Currently only lowercase letters, numbers
            and underscores are allowed in the tag.
        status (google.ads.googleads.v21.enums.types.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus):
            The status of the conversion custom variable
            for conversion event accrual.
        owner_customer (str):
            Output only. The resource name of the
            customer that owns the conversion custom
            variable.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    tag: str = proto.Field(
        proto.STRING,
        number=4,
    )
    status: (
        conversion_custom_variable_status.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=conversion_custom_variable_status.ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus,
    )
    owner_customer: str = proto.Field(
        proto.STRING,
        number=6,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "UserListSizeRangeEnum",
    },
)


class UserListSizeRangeEnum(proto.Message):
    r"""Size range in terms of number of users of a UserList."""

    class UserListSizeRange(proto.Enum):
        r"""Enum containing possible user list size ranges.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            LESS_THAN_FIVE_HUNDRED (2):
                User list has less than 500 users.
            LESS_THAN_ONE_THOUSAND (3):
                User list has number of users in range of 500
                to 1000.
            ONE_THOUSAND_TO_TEN_THOUSAND (4):
                User list has number of users in range of
                1000 to 10000.
            TEN_THOUSAND_TO_FIFTY_THOUSAND (5):
                User list has number of users in range of
                10000 to 50000.
            FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND (6):
                User list has number of users in range of
                50000 to 100000.
            ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND (7):
                User list has number of users in range of
                100000 to 300000.
            THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND (8):
                User list has number of users in range of
                300000 to 500000.
            FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION (9):
                User list has number of users in range of
                500000 to 1 million.
            ONE_MILLION_TO_TWO_MILLION (10):
                User list has number of users in range of 1
                to 2 millions.
            TWO_MILLION_TO_THREE_MILLION (11):
                User list has number of users in range of 2
                to 3 millions.
            THREE_MILLION_TO_FIVE_MILLION (12):
                User list has number of users in range of 3
                to 5 millions.
            FIVE_MILLION_TO_TEN_MILLION (13):
                User list has number of users in range of 5
                to 10 millions.
            TEN_MILLION_TO_TWENTY_MILLION (14):
                User list has number of users in range of 10
                to 20 millions.
            TWENTY_MILLION_TO_THIRTY_MILLION (15):
                User list has number of users in range of 20
                to 30 millions.
            THIRTY_MILLION_TO_FIFTY_MILLION (16):
                User list has number of users in range of 30
                to 50 millions.
            OVER_FIFTY_MILLION (17):
                User list has over 50 million users.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LESS_THAN_FIVE_HUNDRED = 2
        LESS_THAN_ONE_THOUSAND = 3
        ONE_THOUSAND_TO_TEN_THOUSAND = 4
        TEN_THOUSAND_TO_FIFTY_THOUSAND = 5
        FIFTY_THOUSAND_TO_ONE_HUNDRED_THOUSAND = 6
        ONE_HUNDRED_THOUSAND_TO_THREE_HUNDRED_THOUSAND = 7
        THREE_HUNDRED_THOUSAND_TO_FIVE_HUNDRED_THOUSAND = 8
        FIVE_HUNDRED_THOUSAND_TO_ONE_MILLION = 9
        ONE_MILLION_TO_TWO_MILLION = 10
        TWO_MILLION_TO_THREE_MILLION = 11
        THREE_MILLION_TO_FIVE_MILLION = 12
        FIVE_MILLION_TO_TEN_MILLION = 13
        TEN_MILLION_TO_TWENTY_MILLION = 14
        TWENTY_MILLION_TO_THIRTY_MILLION = 15
        THIRTY_MILLION_TO_FIFTY_MILLION = 16
        OVER_FIFTY_MILLION = 17


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a lead form and a lead form extension for a campaign.

Run add_campaigns.py to create a campaign.
"""


import argparse
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.asset_service import AssetOperation
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.common.types.asset_types import LeadFormAsset
from google.ads.googleads.v22.common.types.asset_types import LeadFormField
from google.ads.googleads.v22.common.types.asset_types import (
    LeadFormDeliveryMethod,
)
from google.ads.googleads.v22.services.types.campaign_asset_service import (
    CampaignAssetOperation,
)
from google.ads.googleads.v22.resources.types.campaign_asset import (
    CampaignAsset,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    """Creates a lead form and lead form extension for the given campaign.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The Google Ads customer ID.
        campaign_id: The ID for a Campaign belonging to the given customer.
    """
    lead_form_asset_resource_name: str = create_lead_form_asset(
        client, customer_id
    )
    create_lead_form_campaign_asset(
        client, customer_id, campaign_id, lead_form_asset_resource_name
    )


# [START add_lead_form_asset]
def create_lead_form_asset(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a lead form asset using the given customer ID.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The Google Ads customer ID.

    Returns:
        A str of the resource name for the newly created lead form asset.
    """
    asset_service = client.get_service("AssetService")
    asset_operation: AssetOperation = client.get_type("AssetOperation")
    asset: Asset = asset_operation.create
    asset.name = f"Interplanetary Cruise #{uuid4()} Lead Form"
    asset.final_urls.append("http://example.com/jupiter")

    # Creates a new LeadFormAsset instance.
    lead_form_asset: LeadFormAsset = asset.lead_form_asset

    # Specify the details of the extension that the users will see.
    lead_form_asset.call_to_action_type = (
        client.enums.LeadFormCallToActionTypeEnum.BOOK_NOW
    )
    lead_form_asset.call_to_action_description = "Latest trip to Jupiter!"

    # Define the form details.
    lead_form_asset.business_name = "Interplanetary Cruise"
    lead_form_asset.headline = "Trip to Jupiter"
    lead_form_asset.description = (
        "Our latest trip to Jupiter is now open for booking."
    )
    lead_form_asset.privacy_policy_url = "http://example.com/privacy"

    # Define the fields to be displayed to the user.
    input_type_enum = client.enums.LeadFormFieldUserInputTypeEnum
    lead_form_field_1: LeadFormField = client.get_type("LeadFormField")
    lead_form_field_1.input_type = input_type_enum.FULL_NAME
    lead_form_asset.fields.append(lead_form_field_1)

    lead_form_field_2: LeadFormField = client.get_type("LeadFormField")
    lead_form_field_2.input_type = input_type_enum.EMAIL
    lead_form_asset.fields.append(lead_form_field_2)

    lead_form_field_3: LeadFormField = client.get_type("LeadFormField")
    lead_form_field_3.input_type = input_type_enum.PHONE_NUMBER
    lead_form_asset.fields.append(lead_form_field_3)

    lead_form_field_4: LeadFormField = client.get_type("LeadFormField")
    lead_form_field_4.input_type = input_type_enum.PREFERRED_CONTACT_TIME
    lead_form_field_4.single_choice_answers.answers.extend(
        ["Before 9 AM", "Anytime", "After 5 PM"]
    )
    lead_form_asset.fields.append(lead_form_field_4)

    # Optional: You can also specify a background image asset.
    # To upload an asset, see misc/upload_image.py.
    # lead_form_asset.background_image_asset = "INSERT_IMAGE_ASSET_HERE"

    # Optional: Define the response page after the user signs up on the form.
    lead_form_asset.post_submit_headline = "Thanks for signing up!"
    lead_form_asset.post_submit_description = (
        "We will reach out to you shortly. Visit our website to see past trip "
        "details."
    )
    lead_form_asset.post_submit_call_to_action_type = (
        client.enums.LeadFormPostSubmitCallToActionTypeEnum.VISIT_SITE
    )

    # Optional: Display a custom disclosure that displays along with the Google
    # disclaimer on the form.
    lead_form_asset.custom_disclosure = (
        "Trip may get cancelled due to meteor shower"
    )

    # Optional: Define a delivery method for the form response. See
    # https://developers.google.com/google-ads/webhook/docs/overview for more
    # details on how to define a webhook.
    delivery_method: LeadFormDeliveryMethod = client.get_type(
        "LeadFormDeliveryMethod"
    )
    delivery_method.webhook.advertiser_webhook_url = (
        "http://example.com/webhook"
    )
    delivery_method.webhook.google_secret = "interplanetary google secret"
    delivery_method.webhook.payload_schema_version = 3
    lead_form_asset.delivery_methods.append(delivery_method)

    asset_service = client.get_service("AssetService")
    response = asset_service.mutate_assets(
        customer_id=customer_id, operations=[asset_operation]
    )
    resource_name: str = response.results[0].resource_name

    print(f"Asset with resource name {resource_name} was created.")

    return resource_name
    # [END add_lead_form_asset]


# [START add_lead_form_asset_1]
def create_lead_form_campaign_asset(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_id: str,
    lead_form_asset_resource_name: str,
) -> None:
    """Creates the lead form campaign asset.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The Google Ads customer ID.
        campaign_id: The ID for a Campaign belonging to the given customer.
        lead_form_asset_resource_name: The resource name of the lead form asset.
    """
    campaign_service = client.get_service("CampaignService")
    campaign_asset_service = client.get_service("CampaignAssetService")
    campaign_asset_operation: CampaignAssetOperation = client.get_type(
        "CampaignAssetOperation"
    )
    campaign_asset: CampaignAsset = campaign_asset_operation.create
    campaign_asset.asset = lead_form_asset_resource_name
    campaign_asset.field_type = client.enums.AssetFieldTypeEnum.LEAD_FORM
    campaign_asset.campaign = campaign_service.campaign_path(
        customer_id, campaign_id
    )

    response = campaign_asset_service.mutate_campaign_assets(
        customer_id=customer_id, operations=[campaign_asset_operation]
    )
    for result in response.results:
        print(
            "Created campaign asset with resource name "
            f'"{result.resource_name}" for campaign with ID {campaign_id}'
        )
        # [END add_lead_form_asset_1]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="This code example creates a lead form and a lead form "
        "asset for a campaign. Run add_campaigns.py to create a "
        "campaign."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i",
        "--campaign_id",
        type=str,
        required=True,
        help="The ID of a Campaign belonging to the given customer.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.campaign_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "DomainCategory",
    },
)


class DomainCategory(proto.Message):
    r"""A category generated automatically by crawling a domain. If a
    campaign uses the DynamicSearchAdsSetting, then domain
    categories will be generated for the domain. The categories can
    be targeted using WebpageConditionInfo. See:
    https://support.google.com/google-ads/answer/2471185


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the domain category.
            Domain category resource names have the form:

            ``customers/{customer_id}/domainCategories/{campaign_id}~{category_base64}~{language_code}``
        campaign (str):
            Output only. The campaign this category is
            recommended for.

            This field is a member of `oneof`_ ``_campaign``.
        category (str):
            Output only. Recommended category for the
            website domain, for example, if you have a
            website about electronics, the categories could
            be "cameras", "televisions", etc.

            This field is a member of `oneof`_ ``_category``.
        language_code (str):
            Output only. The language code specifying the
            language of the website, for example, "en" for
            English. The language can be specified in the
            DynamicSearchAdsSetting required for dynamic
            search ads. This is the language of the pages
            from your website that you want Google Ads to
            find, create ads for, and match searches with.

            This field is a member of `oneof`_ ``_language_code``.
        domain (str):
            Output only. The domain for the website. The
            domain can be specified in the
            DynamicSearchAdsSetting required for dynamic
            search ads.

            This field is a member of `oneof`_ ``_domain``.
        coverage_fraction (float):
            Output only. Fraction of pages on your site
            that this category matches.

            This field is a member of `oneof`_ ``_coverage_fraction``.
        category_rank (int):
            Output only. The position of this category in
            the set of categories. Lower numbers indicate a
            better match for the domain. null indicates not
            recommended.

            This field is a member of `oneof`_ ``_category_rank``.
        has_children (bool):
            Output only. Indicates whether this category
            has sub-categories.

            This field is a member of `oneof`_ ``_has_children``.
        recommended_cpc_bid_micros (int):
            Output only. The recommended cost per click
            for the category.

            This field is a member of `oneof`_ ``_recommended_cpc_bid_micros``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    category: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    domain: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    coverage_fraction: float = proto.Field(
        proto.DOUBLE,
        number=14,
        optional=True,
    )
    category_rank: int = proto.Field(
        proto.INT64,
        number=15,
        optional=True,
    )
    has_children: bool = proto.Field(
        proto.BOOL,
        number=16,
        optional=True,
    )
    recommended_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=17,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "SettingErrorEnum",
    },
)


class SettingErrorEnum(proto.Message):
    r"""Container for enum describing possible setting errors."""

    class SettingError(proto.Enum):
        r"""Enum describing possible setting errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            SETTING_TYPE_IS_NOT_AVAILABLE (3):
                The campaign setting is not available for
                this Google Ads account.
            SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN (4):
                The setting is not compatible with the
                campaign.
            TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP (5):
                The supplied TargetingSetting contains an
                invalid CriterionTypeGroup. See
                CriterionTypeGroup documentation for
                CriterionTypeGroups allowed in Campaign or
                AdGroup TargetingSettings.
            TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL (6):
                TargetingSetting must not explicitly set any of the
                Demographic CriterionTypeGroups (AGE_RANGE, GENDER, PARENT,
                INCOME_RANGE) to false (it's okay to not set them at all, in
                which case the system will set them to true automatically).
            TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP (7):
                TargetingSetting cannot change any of the Demographic
                CriterionTypeGroups (AGE_RANGE, GENDER, PARENT,
                INCOME_RANGE) from true to false.
            DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT (8):
                At least one feed id should be present.
            DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME (9):
                The supplied DynamicSearchAdsSetting contains
                an invalid domain name.
            DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME (10):
                The supplied DynamicSearchAdsSetting contains
                a subdomain name.
            DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE (11):
                The supplied DynamicSearchAdsSetting contains
                an invalid language code.
            TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN (12):
                TargetingSettings in search campaigns should
                not have CriterionTypeGroup.PLACEMENT set to
                targetAll.
            SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN (20):
                The setting value is not compatible with the
                campaign type.
            BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING (21):
                Switching from observation setting to
                targeting setting is not allowed for Customer
                Match lists. See
                https://support.google.com/google-ads/answer/6299717.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SETTING_TYPE_IS_NOT_AVAILABLE = 3
        SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN = 4
        TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP = 5
        TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL = (
            6
        )
        TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP = (
            7
        )
        DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT = 8
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME = 9
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME = 10
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE = 11
        TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN = 12
        SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN = 20
        BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING = (
            21
        )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "FieldErrorEnum",
    },
)


class FieldErrorEnum(proto.Message):
    r"""Container for enum describing possible field errors."""

    class FieldError(proto.Enum):
        r"""Enum describing possible field errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            REQUIRED (2):
                The required field was not present.
            IMMUTABLE_FIELD (3):
                The field attempted to be mutated is
                immutable.
            INVALID_VALUE (4):
                The field's value is invalid.
            VALUE_MUST_BE_UNSET (5):
                The field cannot be set.
            REQUIRED_NONEMPTY_LIST (6):
                The required repeated field was empty.
            FIELD_CANNOT_BE_CLEARED (7):
                The field cannot be cleared.
            BLOCKED_VALUE (9):
                The field's value is on a deny-list for this
                field.
            FIELD_CAN_ONLY_BE_CLEARED (10):
                The field's value cannot be modified, except
                for clearing.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        REQUIRED = 2
        IMMUTABLE_FIELD = 3
        INVALID_VALUE = 4
        VALUE_MUST_BE_UNSET = 5
        REQUIRED_NONEMPTY_LIST = 6
        FIELD_CANNOT_BE_CLEARED = 7
        BLOCKED_VALUE = 9
        FIELD_CAN_ONLY_BE_CLEARED = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.resources.types import campaign_lifecycle_goal
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "ConfigureCampaignLifecycleGoalsRequest",
        "CampaignLifecycleGoalOperation",
        "ConfigureCampaignLifecycleGoalsResponse",
        "ConfigureCampaignLifecycleGoalsResult",
    },
)


class ConfigureCampaignLifecycleGoalsRequest(proto.Message):
    r"""Request message for
    [CampaignLifecycleGoalService.configureCampaignLifecycleGoals][].

    Attributes:
        customer_id (str):
            Required. The ID of the customer performing
            the upload.
        operation (google.ads.googleads.v19.services.types.CampaignLifecycleGoalOperation):
            Required. The operation to perform campaign
            lifecycle goal update.
        validate_only (bool):
            Optional. If true, the request is validated
            but not executed. Only errors are returned, not
            results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "CampaignLifecycleGoalOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="CampaignLifecycleGoalOperation",
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CampaignLifecycleGoalOperation(proto.Message):
    r"""A single operation on a campaign lifecycle goal.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. FieldMask that determines which
            resource fields are modified in an update.
        create (google.ads.googleads.v19.resources.types.CampaignLifecycleGoal):
            Create operation: Create a new campaign
            lifecycle goal. The campaign field should be set
            for this operation.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.CampaignLifecycleGoal):
            Update operation: Update an existing campaign
            lifecycle goal. The campaign field should not be
            set for this operation.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    create: campaign_lifecycle_goal.CampaignLifecycleGoal = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=campaign_lifecycle_goal.CampaignLifecycleGoal,
    )
    update: campaign_lifecycle_goal.CampaignLifecycleGoal = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="operation",
        message=campaign_lifecycle_goal.CampaignLifecycleGoal,
    )


class ConfigureCampaignLifecycleGoalsResponse(proto.Message):
    r"""Response message for
    [CampaignLifecycleGoalService.configureCampaignLifecycleGoals][].

    Attributes:
        result (google.ads.googleads.v19.services.types.ConfigureCampaignLifecycleGoalsResult):
            Result for the campaign lifecycle goal
            configuration.
    """

    result: "ConfigureCampaignLifecycleGoalsResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ConfigureCampaignLifecycleGoalsResult",
    )


class ConfigureCampaignLifecycleGoalsResult(proto.Message):
    r"""The result for the campaign lifecycle goal configuration.

    Attributes:
        resource_name (str):
            Returned for the successful operation.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AccessRoleEnum",
    },
)


class AccessRoleEnum(proto.Message):
    r"""Container for enum describing possible access role for user."""

    class AccessRole(proto.Enum):
        r"""Possible access role of a user.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ADMIN (2):
                Owns its account and can control the addition
                of other users.
            STANDARD (3):
                Can modify campaigns, but can't affect other
                users.
            READ_ONLY (4):
                Can view campaigns and account changes, but
                cannot make edits.
            EMAIL_ONLY (5):
                Role for \"email only\" access. Represents an
                email recipient rather than a true User entity.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ADMIN = 2
        STANDARD = 3
        READ_ONLY = 4
        EMAIL_ONLY = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import policy_topic_entry_type
from google.ads.googleads.v19.enums.types import (
    policy_topic_evidence_destination_mismatch_url_type,
)
from google.ads.googleads.v19.enums.types import (
    policy_topic_evidence_destination_not_working_device,
)
from google.ads.googleads.v19.enums.types import (
    policy_topic_evidence_destination_not_working_dns_error_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.common",
    marshal="google.ads.googleads.v19",
    manifest={
        "PolicyViolationKey",
        "PolicyValidationParameter",
        "PolicyTopicEntry",
        "PolicyTopicEvidence",
        "PolicyTopicConstraint",
    },
)


class PolicyViolationKey(proto.Message):
    r"""Key of the violation. The key is used for referring to a
    violation when filing an exemption request.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        policy_name (str):
            Unique ID of the violated policy.

            This field is a member of `oneof`_ ``_policy_name``.
        violating_text (str):
            The text that violates the policy if
            specified. Otherwise, refers to the policy in
            general (for example, when requesting to be
            exempt from the whole policy). If not specified
            for criterion exemptions, the whole policy is
            implied. Must be specified for ad exemptions.

            This field is a member of `oneof`_ ``_violating_text``.
    """

    policy_name: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    violating_text: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class PolicyValidationParameter(proto.Message):
    r"""Parameter for controlling how policy exemption is done.

    Attributes:
        ignorable_policy_topics (MutableSequence[str]):
            The list of policy topics that should not
            cause a PolicyFindingError to be reported. This
            field is currently only compatible with Enhanced
            Text Ad. It corresponds to the
            PolicyTopicEntry.topic field.

            Resources violating these policies will be
            saved, but will not be eligible to serve. They
            may begin serving at a later time due to a
            change in policies, re-review of the resource,
            or a change in advertiser certificates.
        exempt_policy_violation_keys (MutableSequence[google.ads.googleads.v19.common.types.PolicyViolationKey]):
            The list of policy violation keys that should not cause a
            PolicyViolationError to be reported. Not all policy
            violations are exemptable, refer to the is_exemptible field
            in the returned PolicyViolationError.

            Resources violating these polices will be saved, but will
            not be eligible to serve. They may begin serving at a later
            time due to a change in policies, re-review of the resource,
            or a change in advertiser certificates.
    """

    ignorable_policy_topics: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    exempt_policy_violation_keys: MutableSequence["PolicyViolationKey"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="PolicyViolationKey",
        )
    )


class PolicyTopicEntry(proto.Message):
    r"""Policy finding attached to a resource (for example, alcohol
    policy associated with a site that sells alcohol).

    Each PolicyTopicEntry has a topic that indicates the specific
    ads policy the entry is about and a type to indicate the effect
    that the entry will have on serving. It may optionally have one
    or more evidences that indicate the reason for the finding. It
    may also optionally have one or more constraints that provide
    details about how serving may be restricted.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        topic (str):
            Policy topic this finding refers to. For example, "ALCOHOL",
            "TRADEMARKS_IN_AD_TEXT", or "DESTINATION_NOT_WORKING". The
            set of possible policy topics is not fixed for a particular
            API version and may change at any time.

            This field is a member of `oneof`_ ``_topic``.
        type_ (google.ads.googleads.v19.enums.types.PolicyTopicEntryTypeEnum.PolicyTopicEntryType):
            Describes the negative or positive effect
            this policy will have on serving.
        evidences (MutableSequence[google.ads.googleads.v19.common.types.PolicyTopicEvidence]):
            Additional information that explains policy
            finding (for example, the brand name for a
            trademark finding).
        constraints (MutableSequence[google.ads.googleads.v19.common.types.PolicyTopicConstraint]):
            Indicates how serving of this resource may be
            affected (for example, not serving in a
            country).
    """

    topic: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    type_: (
        policy_topic_entry_type.PolicyTopicEntryTypeEnum.PolicyTopicEntryType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_topic_entry_type.PolicyTopicEntryTypeEnum.PolicyTopicEntryType,
    )
    evidences: MutableSequence["PolicyTopicEvidence"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="PolicyTopicEvidence",
    )
    constraints: MutableSequence["PolicyTopicConstraint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="PolicyTopicConstraint",
    )


class PolicyTopicEvidence(proto.Message):
    r"""Additional information that explains a policy finding.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        website_list (google.ads.googleads.v19.common.types.PolicyTopicEvidence.WebsiteList):
            List of websites linked with this resource.

            This field is a member of `oneof`_ ``value``.
        text_list (google.ads.googleads.v19.common.types.PolicyTopicEvidence.TextList):
            List of evidence found in the text of a
            resource.

            This field is a member of `oneof`_ ``value``.
        language_code (str):
            The language the resource was detected to be
            written in. This is an IETF language tag such as
            "en-US".

            This field is a member of `oneof`_ ``value``.
        destination_text_list (google.ads.googleads.v19.common.types.PolicyTopicEvidence.DestinationTextList):
            The text in the destination of the resource
            that is causing a policy finding.

            This field is a member of `oneof`_ ``value``.
        destination_mismatch (google.ads.googleads.v19.common.types.PolicyTopicEvidence.DestinationMismatch):
            Mismatch between the destinations of a
            resource's URLs.

            This field is a member of `oneof`_ ``value``.
        destination_not_working (google.ads.googleads.v19.common.types.PolicyTopicEvidence.DestinationNotWorking):
            Details when the destination is returning an
            HTTP error code or isn't functional in all
            locations for commonly used devices.

            This field is a member of `oneof`_ ``value``.
    """

    class TextList(proto.Message):
        r"""A list of fragments of text that violated a policy.

        Attributes:
            texts (MutableSequence[str]):
                The fragments of text from the resource that
                caused the policy finding.
        """

        texts: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class WebsiteList(proto.Message):
        r"""A list of websites that caused a policy finding. Used for
        ONE_WEBSITE_PER_AD_GROUP policy topic, for example. In case there
        are more than five websites, only the top five (those that appear in
        resources the most) will be listed here.

        Attributes:
            websites (MutableSequence[str]):
                Websites that caused the policy finding.
        """

        websites: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class DestinationTextList(proto.Message):
        r"""A list of strings found in a destination page that caused a
        policy finding.

        Attributes:
            destination_texts (MutableSequence[str]):
                List of text found in the resource's
                destination page.
        """

        destination_texts: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class DestinationMismatch(proto.Message):
        r"""Evidence of mismatches between the URLs of a resource.

        Attributes:
            url_types (MutableSequence[google.ads.googleads.v19.enums.types.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType]):
                The set of URLs that did not match each
                other.
        """

        url_types: MutableSequence[
            policy_topic_evidence_destination_mismatch_url_type.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType
        ] = proto.RepeatedField(
            proto.ENUM,
            number=1,
            enum=policy_topic_evidence_destination_mismatch_url_type.PolicyTopicEvidenceDestinationMismatchUrlTypeEnum.PolicyTopicEvidenceDestinationMismatchUrlType,
        )

    class DestinationNotWorking(proto.Message):
        r"""Evidence details when the destination is returning an HTTP
        error code or isn't functional in all locations for commonly
        used devices.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            expanded_url (str):
                The full URL that didn't work.

                This field is a member of `oneof`_ ``_expanded_url``.
            device (google.ads.googleads.v19.enums.types.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice):
                The type of device that failed to load the
                URL.
            last_checked_date_time (str):
                The time the URL was last checked.
                The format is "YYYY-MM-DD HH:MM:SS".
                Examples: "2018-03-05 09:15:00" or "2018-02-01
                14:34:30".

                This field is a member of `oneof`_ ``_last_checked_date_time``.
            dns_error_type (google.ads.googleads.v19.enums.types.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType):
                The type of DNS error.

                This field is a member of `oneof`_ ``reason``.
            http_error_code (int):
                The HTTP error code.

                This field is a member of `oneof`_ ``reason``.
        """

        expanded_url: str = proto.Field(
            proto.STRING,
            number=7,
            optional=True,
        )
        device: (
            policy_topic_evidence_destination_not_working_device.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice
        ) = proto.Field(
            proto.ENUM,
            number=4,
            enum=policy_topic_evidence_destination_not_working_device.PolicyTopicEvidenceDestinationNotWorkingDeviceEnum.PolicyTopicEvidenceDestinationNotWorkingDevice,
        )
        last_checked_date_time: str = proto.Field(
            proto.STRING,
            number=8,
            optional=True,
        )
        dns_error_type: (
            policy_topic_evidence_destination_not_working_dns_error_type.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            oneof="reason",
            enum=policy_topic_evidence_destination_not_working_dns_error_type.PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum.PolicyTopicEvidenceDestinationNotWorkingDnsErrorType,
        )
        http_error_code: int = proto.Field(
            proto.INT64,
            number=6,
            oneof="reason",
        )

    website_list: WebsiteList = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="value",
        message=WebsiteList,
    )
    text_list: TextList = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="value",
        message=TextList,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=9,
        oneof="value",
    )
    destination_text_list: DestinationTextList = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="value",
        message=DestinationTextList,
    )
    destination_mismatch: DestinationMismatch = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="value",
        message=DestinationMismatch,
    )
    destination_not_working: DestinationNotWorking = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="value",
        message=DestinationNotWorking,
    )


class PolicyTopicConstraint(proto.Message):
    r"""Describes the effect on serving that a policy topic entry
    will have.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        country_constraint_list (google.ads.googleads.v19.common.types.PolicyTopicConstraint.CountryConstraintList):
            Countries where the resource cannot serve.

            This field is a member of `oneof`_ ``value``.
        reseller_constraint (google.ads.googleads.v19.common.types.PolicyTopicConstraint.ResellerConstraint):
            Reseller constraint.

            This field is a member of `oneof`_ ``value``.
        certificate_missing_in_country_list (google.ads.googleads.v19.common.types.PolicyTopicConstraint.CountryConstraintList):
            Countries where a certificate is required for
            serving.

            This field is a member of `oneof`_ ``value``.
        certificate_domain_mismatch_in_country_list (google.ads.googleads.v19.common.types.PolicyTopicConstraint.CountryConstraintList):
            Countries where the resource's domain is not
            covered by the certificates associated with it.

            This field is a member of `oneof`_ ``value``.
    """

    class CountryConstraintList(proto.Message):
        r"""A list of countries where a resource's serving is
        constrained.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            total_targeted_countries (int):
                Total number of countries targeted by the
                resource.

                This field is a member of `oneof`_ ``_total_targeted_countries``.
            countries (MutableSequence[google.ads.googleads.v19.common.types.PolicyTopicConstraint.CountryConstraint]):
                Countries in which serving is restricted.
        """

        total_targeted_countries: int = proto.Field(
            proto.INT32,
            number=3,
            optional=True,
        )
        countries: MutableSequence[
            "PolicyTopicConstraint.CountryConstraint"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="PolicyTopicConstraint.CountryConstraint",
        )

    class ResellerConstraint(proto.Message):
        r"""Indicates that a policy topic was constrained due to
        disapproval of the website for reseller purposes.

        """

    class CountryConstraint(proto.Message):
        r"""Indicates that a resource's ability to serve in a particular
        country is constrained.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            country_criterion (str):
                Geo target constant resource name of the
                country in which serving is constrained.

                This field is a member of `oneof`_ ``_country_criterion``.
        """

        country_criterion: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )

    country_constraint_list: CountryConstraintList = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="value",
        message=CountryConstraintList,
    )
    reseller_constraint: ResellerConstraint = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="value",
        message=ResellerConstraint,
    )
    certificate_missing_in_country_list: CountryConstraintList = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="value",
        message=CountryConstraintList,
    )
    certificate_domain_mismatch_in_country_list: CountryConstraintList = (
        proto.Field(
            proto.MESSAGE,
            number=4,
            oneof="value",
            message=CountryConstraintList,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a combination user list.

The list will contain users that are present on any one of the provided user
lists.
"""

import argparse
import sys
from uuid import uuid4
from typing import List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.user_lists import (
    LogicalUserListOperandInfo,
    UserListLogicalRuleInfo,
)
from google.ads.googleads.v22.resources.types.user_list import UserList
from google.ads.googleads.v22.services.services.user_list_service import (
    UserListServiceClient,
)
from google.ads.googleads.v22.services.types.user_list_service import (
    UserListOperation,
    MutateUserListsResponse,
)


# [START add_logical_user_list]
def main(
    client: GoogleAdsClient, customer_id: str, user_list_ids: List[str]
) -> None:
    """Creates a combination user list.

    Args:
        client: The Google Ads client.
        customer_id: The customer ID for which to add the user list.
        user_list_ids: A list of user list IDs to logically combine.
    """
    # Get the UserListService client.
    user_list_service: UserListServiceClient = client.get_service(
        "UserListService"
    )

    # Add each of the provided list IDs to a list of rule operands specifying
    # which lists the operator should target.
    logical_user_list_operand_info_list: List[LogicalUserListOperandInfo] = []
    for user_list_id in user_list_ids:
        logical_user_list_operand_info: LogicalUserListOperandInfo = (
            client.get_type("LogicalUserListOperandInfo")
        )
        logical_user_list_operand_info.user_list = (
            user_list_service.user_list_path(customer_id, user_list_id)
        )
        logical_user_list_operand_info_list.append(
            logical_user_list_operand_info
        )

    # Create a UserListOperation and populate the UserList.
    user_list_operation: UserListOperation = client.get_type(
        "UserListOperation"
    )
    user_list: UserList = user_list_operation.create
    user_list.name = f"My combination list of other user lists #{uuid4()}"
    # Create a UserListLogicalRuleInfo specifying that a user should be added to
    # the new list if they are present in any of the provided lists.
    user_list_logical_rule_info: UserListLogicalRuleInfo = client.get_type(
        "UserListLogicalRuleInfo"
    )
    # Using ANY means that a user should be added to the combined list if they
    # are present on any of the lists targeted in the
    # LogicalUserListOperandInfo. Use ALL to add users present on all of the
    # provided lists or NONE to add users that aren't present on any of the
    # targeted lists.
    user_list_logical_rule_info.operator = (
        client.enums.UserListLogicalRuleOperatorEnum.ANY
    )
    user_list_logical_rule_info.rule_operands.extend(
        logical_user_list_operand_info_list
    )
    user_list.logical_user_list.rules.append(user_list_logical_rule_info)

    # Issue a mutate request to add the user list, then print the results.
    response: MutateUserListsResponse = user_list_service.mutate_user_lists(
        customer_id=customer_id, operations=[user_list_operation]
    )
    print(
        "Created logical user list with resource name "
        f"'{response.results[0].resource_name}.'"
    )
    # [END add_logical_user_list]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Creates a combination user list containing users that are "
        "present on any one of the provided user lists."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-l",
        "--user_list_ids",
        nargs="+",
        type=str,
        required=True,
        help="The user list IDs logically combine.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.user_list_ids)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "ExperimentErrorEnum",
    },
)


class ExperimentErrorEnum(proto.Message):
    r"""Container for enum describing possible experiment error."""

    class ExperimentError(proto.Enum):
        r"""Enum describing possible experiment errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_SET_START_DATE_IN_PAST (2):
                The start date of an experiment cannot be set
                in the past. Use a start date in the future.
            END_DATE_BEFORE_START_DATE (3):
                The end date of an experiment is before its
                start date. Use an end date after the start
                date.
            START_DATE_TOO_FAR_IN_FUTURE (4):
                The start date of an experiment is too far in
                the future. Use a start date no more than 1 year
                in the future.
            DUPLICATE_EXPERIMENT_NAME (5):
                The experiment has the same name as an
                existing active experiment.
            CANNOT_MODIFY_REMOVED_EXPERIMENT (6):
                Experiments can only be modified when they
                are ENABLED.
            START_DATE_ALREADY_PASSED (7):
                The start date of an experiment cannot be
                modified if the existing start date has already
                passed.
            CANNOT_SET_END_DATE_IN_PAST (8):
                The end date of an experiment cannot be set
                in the past.
            CANNOT_SET_STATUS_TO_REMOVED (9):
                The status of an experiment cannot be set to
                REMOVED.
            CANNOT_MODIFY_PAST_END_DATE (10):
                The end date of an expired experiment cannot
                be modified.
            INVALID_STATUS (11):
                The status is invalid.
            INVALID_CAMPAIGN_CHANNEL_TYPE (12):
                Experiment arm contains campaigns with
                invalid advertising channel type.
            OVERLAPPING_MEMBERS_AND_DATE_RANGE (13):
                A pair of trials share members and have
                overlapping date ranges.
            INVALID_TRIAL_ARM_TRAFFIC_SPLIT (14):
                Experiment arm contains invalid traffic
                split.
            TRAFFIC_SPLIT_OVERLAPPING (15):
                Experiment contains trial arms with
                overlapping traffic split.
            SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR (16):
                The total traffic split of trial arms is not
                equal to 100.
            CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START (17):
                Traffic split related settings (like traffic
                share bounds) can't be modified after the
                experiment has started.
            EXPERIMENT_NOT_FOUND (18):
                The experiment could not be found.
            EXPERIMENT_NOT_YET_STARTED (19):
                Experiment has not begun.
            CANNOT_HAVE_MULTIPLE_CONTROL_ARMS (20):
                The experiment cannot have more than one
                control arm.
            IN_DESIGN_CAMPAIGNS_NOT_SET (21):
                The experiment doesn't set in-design
                campaigns.
            CANNOT_SET_STATUS_TO_GRADUATED (22):
                Clients must use the graduate action to
                graduate experiments and cannot set the status
                to GRADUATED directly.
            CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET (23):
                Cannot use shared budget on base campaign
                when scheduling an experiment.
            CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET (24):
                Cannot use custom budget on base campaign
                when scheduling an experiment.
            STATUS_TRANSITION_INVALID (25):
                Invalid status transition.
            DUPLICATE_EXPERIMENT_CAMPAIGN_NAME (26):
                The experiment campaign name conflicts with a
                pre-existing campaign.
            CANNOT_REMOVE_IN_CREATION_EXPERIMENT (27):
                Cannot remove in creation experiments.
            CANNOT_ADD_CAMPAIGN_WITH_DEPRECATED_AD_TYPES (28):
                Cannot add campaign with deprecated ad types. Deprecated ad
                types: ENHANCED_DISPLAY, GALLERY, GMAIL, KEYWORDLESS, TEXT.
            CANNOT_ENABLE_SYNC_FOR_UNSUPPORTED_EXPERIMENT_TYPE (29):
                Sync can only be enabled for supported experiment types.
                Supported experiment types: SEARCH_CUSTOM, DISPLAY_CUSTOM,
                DISPLAY_AUTOMATED_BIDDING_STRATEGY,
                SEARCH_AUTOMATED_BIDDING_STRATEGY.
            INVALID_DURATION_FOR_AN_EXPERIMENT (30):
                Experiment length cannot be longer than max
                length.
            MISSING_EU_POLITICAL_ADVERTISING_SELF_DECLARATION (31):
                The experiment's campaigns must self-declare
                whether they contain political advertising that
                targets the European Union.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_SET_START_DATE_IN_PAST = 2
        END_DATE_BEFORE_START_DATE = 3
        START_DATE_TOO_FAR_IN_FUTURE = 4
        DUPLICATE_EXPERIMENT_NAME = 5
        CANNOT_MODIFY_REMOVED_EXPERIMENT = 6
        START_DATE_ALREADY_PASSED = 7
        CANNOT_SET_END_DATE_IN_PAST = 8
        CANNOT_SET_STATUS_TO_REMOVED = 9
        CANNOT_MODIFY_PAST_END_DATE = 10
        INVALID_STATUS = 11
        INVALID_CAMPAIGN_CHANNEL_TYPE = 12
        OVERLAPPING_MEMBERS_AND_DATE_RANGE = 13
        INVALID_TRIAL_ARM_TRAFFIC_SPLIT = 14
        TRAFFIC_SPLIT_OVERLAPPING = 15
        SUM_TRIAL_ARM_TRAFFIC_UNEQUALS_TO_TRIAL_TRAFFIC_SPLIT_DENOMINATOR = 16
        CANNOT_MODIFY_TRAFFIC_SPLIT_AFTER_START = 17
        EXPERIMENT_NOT_FOUND = 18
        EXPERIMENT_NOT_YET_STARTED = 19
        CANNOT_HAVE_MULTIPLE_CONTROL_ARMS = 20
        IN_DESIGN_CAMPAIGNS_NOT_SET = 21
        CANNOT_SET_STATUS_TO_GRADUATED = 22
        CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_SHARED_BUDGET = 23
        CANNOT_CREATE_EXPERIMENT_CAMPAIGN_WITH_CUSTOM_BUDGET = 24
        STATUS_TRANSITION_INVALID = 25
        DUPLICATE_EXPERIMENT_CAMPAIGN_NAME = 26
        CANNOT_REMOVE_IN_CREATION_EXPERIMENT = 27
        CANNOT_ADD_CAMPAIGN_WITH_DEPRECATED_AD_TYPES = 28
        CANNOT_ENABLE_SYNC_FOR_UNSUPPORTED_EXPERIMENT_TYPE = 29
        INVALID_DURATION_FOR_AN_EXPERIMENT = 30
        MISSING_EU_POLITICAL_ADVERTISING_SELF_DECLARATION = 31


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import app_url_operating_system_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "FinalAppUrl",
    },
)


class FinalAppUrl(proto.Message):
    r"""A URL for deep linking into an app for the given operating
    system.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        os_type (google.ads.googleads.v20.enums.types.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType):
            The operating system targeted by this URL.
            Required.
        url (str):
            The app deep link URL. Deep links specify a location in an
            app that corresponds to the content you'd like to show, and
            should be of the form {scheme}://{host_path} The scheme
            identifies which app to open. For your app, you can use a
            custom scheme that starts with the app's name. The host and
            path specify the unique location in the app where your
            content exists. Example: "exampleapp://productid_1234".
            Required.

            This field is a member of `oneof`_ ``_url``.
    """

    os_type: (
        app_url_operating_system_type.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=app_url_operating_system_type.AppUrlOperatingSystemTypeEnum.AppUrlOperatingSystemType,
    )
    url: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "DynamicSearchAdsSearchTermView",
    },
)


class DynamicSearchAdsSearchTermView(proto.Message):
    r"""A dynamic search ads search term view.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the dynamic search ads
            search term view. Dynamic search ads search term view
            resource names have the form:

            ``customers/{customer_id}/dynamicSearchAdsSearchTermViews/{ad_group_id}~{search_term_fingerprint}~{headline_fingerprint}~{landing_page_fingerprint}~{page_url_fingerprint}``
        search_term (str):
            Output only. Search term

            This field is read-only.

            This field is a member of `oneof`_ ``_search_term``.
        headline (str):
            Output only. The dynamically generated
            headline of the Dynamic Search Ad.
            This field is read-only.

            This field is a member of `oneof`_ ``_headline``.
        landing_page (str):
            Output only. The dynamically selected landing
            page URL of the impression.
            This field is read-only.

            This field is a member of `oneof`_ ``_landing_page``.
        page_url (str):
            Output only. The URL of page feed item served
            for the impression.
            This field is read-only.

            This field is a member of `oneof`_ ``_page_url``.
        has_negative_keyword (bool):
            Output only. True if query matches a negative
            keyword.
            This field is read-only.

            This field is a member of `oneof`_ ``_has_negative_keyword``.
        has_matching_keyword (bool):
            Output only. True if query is added to
            targeted keywords.
            This field is read-only.

            This field is a member of `oneof`_ ``_has_matching_keyword``.
        has_negative_url (bool):
            Output only. True if query matches a negative
            url.
            This field is read-only.

            This field is a member of `oneof`_ ``_has_negative_url``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    search_term: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    headline: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    landing_page: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    page_url: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    has_negative_keyword: bool = proto.Field(
        proto.BOOL,
        number=13,
        optional=True,
    )
    has_matching_keyword: bool = proto.Field(
        proto.BOOL,
        number=14,
        optional=True,
    )
    has_negative_url: bool = proto.Field(
        proto.BOOL,
        number=15,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Imports offline call conversion values for calls related to your ads.

To set up a conversion action, run the add_conversion_action.py example.
"""


import argparse
import sys
from typing import Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.conversion_upload_service import (
    ConversionUploadServiceClient,
)
from google.ads.googleads.v22.services.types.conversion_upload_service import (
    CallConversion,
    CustomVariable,
    UploadCallConversionsResponse,
    UploadCallConversionsRequest,
    CallConversionResult,
)

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException


# [START upload_call_conversion]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    conversion_action_id: str,
    caller_id: str,
    call_start_date_time: str,
    conversion_date_time: str,
    conversion_value: float,
    conversion_custom_variable_id: Optional[str],
    conversion_custom_variable_value: Optional[str],
    ad_user_data_consent: Optional[str],
):
    """Imports offline call conversion values for calls related to your ads.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID string.
        conversion_action_id: The ID of the conversion action to upload to.
        caller_id: The caller ID from which this call was placed. Caller ID is
            expected to be in E.164 format with preceding '+' sign,
            e.g. '+18005550100'.
        call_start_date_time: The date and time at which the call occurred. The
            format is 'yyyy-mm-dd hh:mm:ss+|-hh:mm',
            e.g. '2021-01-01 12:32:45-08:00'.
        conversion_date_time: The the date and time of the conversion (should be
            after the click time). The format is 'yyyy-mm-dd hh:mm:ss+|-hh:mm',
            e.g. '2021-01-01 12:32:45-08:00'.
        conversion_value: The conversion value in the desired currency.
        conversion_custom_variable_id: The ID of the conversion custom
            variable to associate with the upload.
        conversion_custom_variable_value: The str value of the conversion custom
            variable to associate with the upload.
        ad_user_data_consent: The consent status for ad user data for all
            members in the job.
    """
    # Get the ConversionUploadService client.
    conversion_upload_service: ConversionUploadServiceClient = (
        client.get_service("ConversionUploadService")
    )

    # Create a call conversion in USD currency.
    call_conversion: CallConversion = client.get_type("CallConversion")
    call_conversion.conversion_action = client.get_service(
        "ConversionActionService"
    ).conversion_action_path(customer_id, conversion_action_id)
    call_conversion.caller_id = caller_id
    call_conversion.call_start_date_time = call_start_date_time
    call_conversion.conversion_date_time = conversion_date_time
    call_conversion.conversion_value = conversion_value
    call_conversion.currency_code = "USD"

    if conversion_custom_variable_id and conversion_custom_variable_value:
        conversion_custom_variable: CustomVariable = client.get_type(
            "CustomVariable"
        )
        conversion_custom_variable.conversion_custom_variable = (
            conversion_custom_variable_id
        )
        conversion_custom_variable.value = conversion_custom_variable_value
        call_conversion.custom_variables.append(conversion_custom_variable)

    # Specifies whether user consent was obtained for the data you are
    # uploading. For more details. see:
    # https://www.google.com/about/company/user-consent-policy
    if ad_user_data_consent:
        call_conversion.consent.ad_user_data = client.enums.ConsentStatusEnum[
            ad_user_data_consent
        ]

    # Issue a request to upload the call conversion.
    # Partial failure MUST be enabled for this request.
    request: UploadCallConversionsRequest = client.get_type(
        "UploadCallConversionsRequest"
    )
    request.customer_id = customer_id
    request.conversions = [call_conversion]
    request.partial_failure = True
    # NOTE: This request only uploads a single conversion, but if you have
    # multiple conversions to upload, it's most efficient to upload them in a
    # single request. See the following for per-request limits for reference:
    # https://developers.google.com/google-ads/api/docs/best-practices/quotas#conversion_upload_service
    upload_call_conversions_response: UploadCallConversionsResponse = (
        conversion_upload_service.upload_call_conversions(request=request)
    )

    # Print any partial errors returned.
    if upload_call_conversions_response.partial_failure_error:
        print(
            "Partial error occurred: "
            f"'{upload_call_conversions_response.partial_failure_error.message}'"
        )

    # Print the result if valid.
    uploaded_call_conversion: CallConversionResult = (
        upload_call_conversions_response.results[0]
    )
    if uploaded_call_conversion.call_start_date_time:
        print(
            "Uploaded call conversion that occurred at "
            f"'{uploaded_call_conversion.call_start_date_time}' "
            f"for caller ID '{uploaded_call_conversion.caller_id}' "
            "to the conversion action with resource name "
            f"'{uploaded_call_conversion.conversion_action}'."
        )
    # [END upload_call_conversion]


if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    parser = argparse.ArgumentParser(
        description="Imports offline call conversion values for calls related "
        "to your ads."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--conversion_action_id",
        type=str,
        required=True,
        help="The ID of the conversion action to upload to.",
    )
    parser.add_argument(
        "-i",
        "--caller_id",
        type=str,
        required=True,
        help="The caller ID from which this call was placed. Caller ID is "
        "expected to be in E.164 format with preceding '+' sign, "
        "e.g. '+18005550100'.",
    )
    parser.add_argument(
        "-s",
        "--call_start_date_time",
        type=str,
        required=True,
        help="The date and time at which the call occurred. The format is "
        "'yyyy-mm-dd hh:mm:ss+|-hh:mm', e.g. '2019-01-01 12:32:45-08:00'.",
    )
    parser.add_argument(
        "-t",
        "--conversion_date_time",
        type=str,
        required=True,
        help="The date and time of the conversion (should be after the "
        "click time). The format is 'yyyy-mm-dd hh:mm:ss+|-hh:mm', e.g. "
        "'2019-01-01 12:32:45-08:00'.",
    )
    parser.add_argument(
        "-v",
        "--conversion_value",
        type=float,
        required=True,
        help="The conversion value in the desired currency.",
    )
    parser.add_argument(
        "-w",
        "--conversion_custom_variable_id",
        type=str,
        help="The ID of the conversion custom variable to associate with the upload.",
    )
    parser.add_argument(
        "-x",
        "--conversion_custom_variable_value",
        type=str,
        help="The value of the conversion custom variable to associate with the upload.",
    )
    parser.add_argument(
        "-d",
        "--ad_user_data_consent",
        type=str,
        choices=[
            e.name
            for e in googleads_client.enums.ConsentStatusEnum
            if e.name not in ("UNSPECIFIED", "UNKNOWN")
        ],
        help=(
            "The data consent status for ad user data for all members in "
            "the job."
        ),
    )
    args = parser.parse_args()

    try:
        main(
            googleads_client,
            args.customer_id,
            args.conversion_action_id,
            args.caller_id,
            args.call_start_date_time,
            args.conversion_date_time,
            args.conversion_value,
            args.conversion_custom_variable_id,
            args.conversion_custom_variable_value,
            args.ad_user_data_consent,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}'' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "MediaUploadErrorEnum",
    },
)


class MediaUploadErrorEnum(proto.Message):
    r"""Container for enum describing possible media uploading
    errors.

    """

    class MediaUploadError(proto.Enum):
        r"""Enum describing possible media uploading errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FILE_TOO_BIG (2):
                The uploaded file is too big.
            UNPARSEABLE_IMAGE (3):
                Image data is unparseable.
            ANIMATED_IMAGE_NOT_ALLOWED (4):
                Animated images are not allowed.
            FORMAT_NOT_ALLOWED (5):
                The image or media bundle format is not
                allowed.
            EXTERNAL_URL_NOT_ALLOWED (6):
                Cannot reference URL external to the media
                bundle.
            INVALID_URL_REFERENCE (7):
                HTML5 ad is trying to reference an asset not
                in .ZIP file.
            MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY (8):
                The media bundle contains no primary entry.
            ANIMATED_VISUAL_EFFECT (9):
                Animation has disallowed visual effects.
            ANIMATION_TOO_LONG (10):
                Animation longer than the allowed 30 second
                limit.
            ASPECT_RATIO_NOT_ALLOWED (11):
                The aspect ratio of the image does not match
                the expected aspect ratios provided in the asset
                spec.
            AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE (12):
                Audio files are not allowed in bundle.
            CMYK_JPEG_NOT_ALLOWED (13):
                CMYK jpegs are not supported.
            FLASH_NOT_ALLOWED (14):
                Flash movies are not allowed.
            FRAME_RATE_TOO_HIGH (15):
                The frame rate of the video is higher than
                the allowed 5fps.
            GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED (16):
                ZIP file from Google Web Designer is not
                published.
            IMAGE_CONSTRAINTS_VIOLATED (17):
                Image constraints are violated, but more details (like
                DIMENSIONS_NOT_ALLOWED or ASPECT_RATIO_NOT_ALLOWED) can not
                be provided. This happens when asset spec contains more than
                one constraint and criteria of different constraints are
                violated.
            INVALID_MEDIA_BUNDLE (18):
                Media bundle data is unrecognizable.
            INVALID_MEDIA_BUNDLE_ENTRY (19):
                There was a problem with one or more of the
                media bundle entries.
            INVALID_MIME_TYPE (20):
                The asset has an invalid mime type.
            INVALID_PATH (21):
                The media bundle contains an invalid asset
                path.
            LAYOUT_PROBLEM (22):
                Image has layout problem.
            MALFORMED_URL (23):
                An asset had a URL reference that is
                malformed per RFC 1738 convention.
            MEDIA_BUNDLE_NOT_ALLOWED (24):
                The uploaded media bundle format is not
                allowed.
            MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE (25):
                The media bundle is not compatible with the
                asset spec product type. (For example, Gmail,
                dynamic remarketing, etc.)
            MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS (26):
                A bundle being uploaded that is incompatible
                with multiple assets for different reasons.
            TOO_MANY_FILES_IN_MEDIA_BUNDLE (27):
                The media bundle contains too many files.
            UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT (28):
                Google Web Designer not created for "Google
                Ads" environment.
            UNSUPPORTED_HTML5_FEATURE (29):
                Unsupported HTML5 feature in HTML5 asset.
            URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT (30):
                URL in HTML5 entry is not SSL compliant.
            VIDEO_FILE_NAME_TOO_LONG (31):
                Video file name is longer than the 50 allowed
                characters.
            VIDEO_MULTIPLE_FILES_WITH_SAME_NAME (32):
                Multiple videos with same name in a bundle.
            VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE (33):
                Videos are not allowed in media bundle.
            CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API (34):
                This type of media cannot be uploaded through
                the Google Ads API.
            DIMENSIONS_NOT_ALLOWED (35):
                The dimensions of the image are not allowed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FILE_TOO_BIG = 2
        UNPARSEABLE_IMAGE = 3
        ANIMATED_IMAGE_NOT_ALLOWED = 4
        FORMAT_NOT_ALLOWED = 5
        EXTERNAL_URL_NOT_ALLOWED = 6
        INVALID_URL_REFERENCE = 7
        MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY = 8
        ANIMATED_VISUAL_EFFECT = 9
        ANIMATION_TOO_LONG = 10
        ASPECT_RATIO_NOT_ALLOWED = 11
        AUDIO_NOT_ALLOWED_IN_MEDIA_BUNDLE = 12
        CMYK_JPEG_NOT_ALLOWED = 13
        FLASH_NOT_ALLOWED = 14
        FRAME_RATE_TOO_HIGH = 15
        GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED = 16
        IMAGE_CONSTRAINTS_VIOLATED = 17
        INVALID_MEDIA_BUNDLE = 18
        INVALID_MEDIA_BUNDLE_ENTRY = 19
        INVALID_MIME_TYPE = 20
        INVALID_PATH = 21
        LAYOUT_PROBLEM = 22
        MALFORMED_URL = 23
        MEDIA_BUNDLE_NOT_ALLOWED = 24
        MEDIA_BUNDLE_NOT_COMPATIBLE_TO_PRODUCT_TYPE = 25
        MEDIA_BUNDLE_REJECTED_BY_MULTIPLE_ASSET_SPECS = 26
        TOO_MANY_FILES_IN_MEDIA_BUNDLE = 27
        UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT = 28
        UNSUPPORTED_HTML5_FEATURE = 29
        URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT = 30
        VIDEO_FILE_NAME_TOO_LONG = 31
        VIDEO_MULTIPLE_FILES_WITH_SAME_NAME = 32
        VIDEO_NOT_ALLOWED_IN_MEDIA_BUNDLE = 33
        CANNOT_UPLOAD_MEDIA_TYPE_THROUGH_API = 34
        DIMENSIONS_NOT_ALLOWED = 35


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "AgeRangeTypeEnum",
    },
)


class AgeRangeTypeEnum(proto.Message):
    r"""Container for enum describing the type of demographic age
    ranges.

    """

    class AgeRangeType(proto.Enum):
        r"""The type of demographic age ranges (for example, between 18
        and 24 years old).

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            AGE_RANGE_18_24 (503001):
                Between 18 and 24 years old.
            AGE_RANGE_25_34 (503002):
                Between 25 and 34 years old.
            AGE_RANGE_35_44 (503003):
                Between 35 and 44 years old.
            AGE_RANGE_45_54 (503004):
                Between 45 and 54 years old.
            AGE_RANGE_55_64 (503005):
                Between 55 and 64 years old.
            AGE_RANGE_65_UP (503006):
                65 years old and beyond.
            AGE_RANGE_UNDETERMINED (503999):
                Undetermined age range.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AGE_RANGE_18_24 = 503001
        AGE_RANGE_25_34 = 503002
        AGE_RANGE_35_44 = 503003
        AGE_RANGE_45_54 = 503004
        AGE_RANGE_55_64 = 503005
        AGE_RANGE_65_UP = 503006
        AGE_RANGE_UNDETERMINED = 503999


__all__ = tuple(sorted(__protobuf__.manifest))
